<?php

namespace Annie\Cause\Providers;

use Illuminate\Support\ServiceProvider;

class CauseServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'cause');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'cause');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerCause();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerCause()
    {
        $this->app->bind('cause', function($app) {
            return new Cause($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Cause', 'Lavalite\Cause\Facades\Cause');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('annie.cause', function ($app) {
            return $this->app->make('Annie\Cause\Cause');
        });

        // Bind Cause to repository
        $this->app->bind(
            'Annie\Cause\Interfaces\CauseRepositoryInterface',
            \Annie\Cause\Repositories\Eloquent\CauseRepository::class
        );        // Bind CauseKindSupport to repository
        $this->app->bind(
            'Annie\Cause\Interfaces\CauseKindSupportRepositoryInterface',
            \Annie\Cause\Repositories\Eloquent\CauseKindSupportRepository::class
        );        // Bind CauseProject to repository
        $this->app->bind(
            'Annie\Cause\Interfaces\CauseProjectRepositoryInterface',
            \Annie\Cause\Repositories\Eloquent\CauseProjectRepository::class
        );

        $this->app->register(\Annie\Cause\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Annie\Cause\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and cause's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/cause.php', 'cause'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeCause::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['annie.cause'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('annie/cause.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/cause')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/cause')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
